
#include <stdlib.h>
#include <stdio.h>
#include <mem.h>

#include "cgdk.h"
#include "timer.h"
#include "svga.h"
#include "ztimer.h"
#include "keyb.h"
#include "mouse.h"

#define BUFFER_SIZE 10000
#define COUNT 10000

#define RETRACE_MAX 100

int getFreq (void)
{
  long time;
  int i;

  time = TIMER_ticks;

  SVGA_wait (1);

  TIMER_start ();

  for (i = 0; i < RETRACE_MAX; i++)
  {
    SVGA_wait (1);
  }

  TIMER_stop ();

  return (((double) TIMER_time * 1000000)/TIMER_FREQ);
}

long lz (void)
{
  LZTimer       ltimer;
  long value;

  ZTimerInit();

  /* Test the long period Zen Timer (we don't check for overflow coz
   * it would take tooooo long!)
   */

  cout << endl;
  ltimer.start();
  getFreq ();
  ltimer.stop();
  cout << "LCount:  " << ltimer.count() << endl;
  cout << "Time:    " << ltimer << " secs\n";
  value = ltimer.count ();
  ltimer.restart();
  return value;
}

extern "C" int SVGA_init (void);

int main (void)
{
  int time;
  double hz;
  int videoMode;

  SVGA_init ();
  KEYB_init ();
  MOUSE_init ();

  time = lz ();
  hz = ((double)1000000 * RETRACE_MAX) / time;
  printf ("Real screen frequency: %f Hz\n", hz);

  if (_argc > 1)
  {
    sscanf (_argv [1], "%i", &videoMode);
    SVGA_setVideoMode (videoMode);
  }

  TIMER_init ();

  time = getFreq ();

  if (_argc > 1)
  {
    SVGA_done ();
  }

  hz = ((double)1000000 * RETRACE_MAX) / time;
  printf ("Timer frequncy: %f Hz\n", (double)TIMER_FREQ);
  printf ("Timer count: %i\n", TIMER_time);
  printf ("Vertical retrace time length: %i mics\n", time);
  printf ("Screen frequency: %f Hz\n", hz);
  printf ("Timer ticks per frame: %f\n", (double)TIMER_time/RETRACE_MAX);
  printf ("Vertical retrace trapped: %i/%i\n", TIMER_retraceTrap, RETRACE_MAX);

  TIMER_done ();

  return 0;
}


/*
int oldmain (void)
{
  int i,j;

  void *buf1;
  void *buf2;

  f1 (1);
  f2 (2);

  CGDK_init ();

  buf1 = _alloc (2*BUFFER_SIZE);
  buf2 = _alloc (2*BUFFER_SIZE);

  TIMER_start ();

  for (i = 0; i < COUNT; i++)
  {
    memcopy2 (buf1, buf2, BUFFER_SIZE);
  }

  printf ("memcopy2: %i\n", TIMER_time);

  TIMER_reset ();
  TIMER_start ();

  for (i = 0; i < COUNT; i++)
  {
    memcopy1 (buf2, buf1, BUFFER_SIZE);
  }

  printf ("memcopy1: %i\n", TIMER_time);

  return 0;
}
*/
/*
extern void mmmemset (void *mem, int c, int count);
#pragma aux mmmemset = \
\
" mov   ah,al"\
" mov   ebx,eax"\
" shl   ebx,16"\
" or    eax,ebx"\
" mov   ebx,ecx"\
" and   ebx,3"\
" shr   ecx,2"\
"shift:"\
" test  edi,01Fh"\
" lea   edi,[edi+4]"\
" jne   shift"\
" repne stosd"\
" mov   ecx,ebx"\
" repne stosb"\
\
parm [edi] [eax] [ecx]\
modify [edi ebx ecx eax ebx]\

extern void _memset (void *mem, int c, int count);
#pragma aux _memset = \
\
" mov   dh,dl"\
" mov   ebx,edx"\
" shl   ebx,16"\
" or    edx,ebx"\
" mov   ebx,ecx"\
" and   ebx,32"\
" shr   ecx,4"\
"shift:"\
" test  edi,01Fh"\
" lea   edi,[edi+4]"\
" jne   shift"\
"store:"\
" cmp   [edi+32],edx"\
" mov   [edi],edx"\
" mov   [edi+4],edx"\
" dec   ecx"\
" mov   [edi+8],edx"\
" mov   [edi+12],edx"\
" je    End"\
" cmp   [edi+48],edx"\
" mov   [edi+16],edx"\
" mov   [edi+20],edx"\
" dec   ecx"\
" mov   [edi+24],edx"\
" mov   [edi+28],edx"\
" lea   edi,[edi+32]"\
" jne   store"\
"End:"\
\
parm [edi] [edx] [ecx]\
modify [edi ebx ecx eax ebx]\
*/
/*
extern void memcopy1 (void *mem1, void *mem2, int count);
#pragma aux memcopy1 = \
"  shr   ecx,2"\
"  je    End"\
"Loop1:"\
"  repne movsd"\
"End:"\
   parm [edi] [esi] [ecx]\
   modify [edi esi ecx]

extern void memcopyx_ (void);
#pragma aux memcopyx_ = \
"  mov   ax,0"\

void memcopyx (void)
{
  memcopyx_ ();
}

extern void memcopy2 (void *mem1, void *mem2, int count);
#pragma aux memcopy2 = \
"  shr   ecx,2"\
"  je    End"\
"Loop1:"\
"  lodsd"\
"  add   eax,4"\
"  add   eax,4"\
"  add   eax,4"\
"  add   eax,4"\
"  add   eax,4"\
"  add   eax,4"\
"  stosd"\
"  call memcopyx"\
"  loop Loop1"\
"End:"\
   parm [edi] [esi] [ecx]\
   modify [edi esi ecx]

int f1 (int i)
{
  printf ("%i",i);
  return 0;
}

int f2 (int i)
{
  printf ("%i",i+1);
  return 0;
}
*/
