
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <mem.h>
#include <conio.h>

#include "cd.h"
#include "keyb.h"
#include "cgdk.h"

void help (void)
{
  printf ("\n"
	  "Press the following keys : Spacebar  to get info\n"
	  "                           A - Z     to select track and play\n"
	  "                           Tab       to pause / resume\n"
	  "                           Backspace to eject\n"
	  "                           Caps Lock to lock / unlock\n"
	  "                           [+] [-]   to adjust volume\n"
	  "\n");
}

void getinfo (void)
{
  int i;

  printf ("                                                             "
	  "                \r");

  CD_getStatus ();

//  if (CD_info.status.doorOpened)
//  {
//    printf ("Door opened.\n");
//    return;
//  }

  if (!CD_inserted)
  {
    printf ("No CD\n");
  }
  else
  {
    for (i = 0; i < CD_info.tracksTotal; i++)
    {

      CD_setTrack (i);
      printf ("%2c)      %02d:%02d:%02d  ", 'a'+i,
              CD_info.trackLength.min, CD_info.trackLength.sec, CD_info.trackLength.frame);
      switch (CD_info.trackType == DATA_TRACK)
      {
	case AUDIO_TRACK:
          printf ("AUDIO TRACK"); break;
        case DATA_TRACK:
	  printf ("DATA TRACK"); break;
        default:
          printf ("UNKNOWN TRACK"); break;
      }
      printf ("\n");
    }
  }
  printf ("\n");
}

#define VOLUME_ADJUST 10
#define VOLUME_MAX 255
#define VOLUME_MIN 0

void volumeUp (void)
{
  TCD_volumeInfo vinfo;

  CD_getVolume (&vinfo);
  if (vinfo.volumeLeft < VOLUME_MAX-VOLUME_ADJUST)
  {
    vinfo.volumeLeft+=VOLUME_ADJUST;
  }
  else
  {
    vinfo.volumeLeft=VOLUME_MAX;
  }
  if (vinfo.volumeRight < VOLUME_MAX-VOLUME_ADJUST)
  {
    vinfo.volumeRight+=VOLUME_ADJUST;
  }
  else
  {
    vinfo.volumeRight=VOLUME_MAX;
  }
  CD_setVolume (&vinfo);
}

void volumeDown (void)
{
  TCD_volumeInfo vinfo;

  CD_getVolume (&vinfo);
  if (vinfo.volumeLeft > VOLUME_MIN+VOLUME_ADJUST)
  {
    vinfo.volumeLeft-=VOLUME_ADJUST;
  }
  else
  {
    vinfo.volumeLeft=VOLUME_MIN;
  }

  if (vinfo.volumeRight > VOLUME_MIN+VOLUME_ADJUST)
  {
    vinfo.volumeRight-=VOLUME_ADJUST;
  }
  else
  {
    vinfo.volumeRight=VOLUME_MIN;
  }

  CD_setVolume (&vinfo);
}

void playTrack (int track)
{
  if (CD_inserted)
  {
    if ((track >= 0) && (track < CD_info.tracksTotal) )
    {
      printf ("\nNow playing track %c)\n",track+'a');
      CD_playTrack (track);
    }
    else
    {
      printf ("\nThe requested track not found.\n\n");
    }
  }
}

void updateStatus (void)
{
  CD_getStatus ();

  printf ("error = %i opened = %i unlocked = %i change = %3i",
	  CD_info.error.errorCode,
	  (CD_info.status.doorOpened != 0),
	  (CD_info.status.doorUnlocked != 0),
	  (CD_mediaChange () ));

  if (CD_info.error.busy)
  {
    CD_getPos ();

    printf (", %02i:%02i:%02i (%02i:%02i:%02i)",
	    CD_playInfo.trackTime.min,CD_playInfo.trackTime.sec,
	    CD_playInfo.trackTime.frame,
	    CD_playInfo.time.min,CD_playInfo.time.sec,
	    CD_playInfo.time.frame);
  }

  printf ("\r");

  fflush (stdout);
}


int main (void)
{
  char c;

  CGDK_init ();
  KEYB_init ();
  CD_init ();

  printf ("\n"
	  "CDROM - Easy! Audio CD player\n"
          "\n%s\n",CGDK_copyright);

  if (CD_installed == CD_INSTALLED)
  {
    printf ("First CDROM drive %c:, total %i drive(s).\n\n", 'A'+CD_info.firstDrive, CD_info.drives);

    CD_audioInit ();

    getinfo ();
    help ();

    while ( (c = KEYB_get ()) != KEY_ESC)
    {
      if (c)
      {
	switch (c)
	{
          case KEY_TAB:
          {
            if (CD_info.error.busy)
            {
              CD_stop ();
            }
            else
            {
              CD_resume ();
            }
            break;
          }
          case KEY_CAPSLOCK:
	  {
	    if (KEYB_flags.capsLock)
	    {
	      CD_lock ();
	    }
	    else
	    {
	      CD_unlock ();
	    }
	    break;
	  }
	  case KEY_BACKSPACE:
	  {
	    CD_eject ();
	    break;
	  }
	  case KEY_SPACE:
          {
            getinfo ();
            help ();
            break;
          }
          case KEY__PLUS:
          {
            volumeUp ();
            break;
	  }
	  case KEY__MINUS:
          {
            volumeDown ();
	    break;
          }
          default:
          {
            playTrack (tolower (KEYB_ascii (c))-'a');
            break;
          }
        }
      }

      updateStatus ();
    }

    return 0;
  }
  else
  {
    printf ("No CDROM or MSCDEX installed\n\n");
    return 1;
  }
}
