
#include <mem.h>
#include <dos.h>

#include "svga.h"
#include "lin.h"
#include "dpmi.h"
#include "mem.h"
#include "cgdk.h"

#include "l256_.h"

#ifdef __TURBOC__
  void L256_memset (void *buf, int value, int count)
  {
    memset (buf, value, count);
  }
#else

  #ifndef CGDK_TRIAL

  extern "C" {

  extern void cdecl L256_movWin2_ (void *offset1, void *offset2, void *win,
			     int srcFormatX, int sizeX, int sizeY);
  extern void cdecl L256_movSpr2_ (void *offset1, void *offset2, void *win,
			     int srcFormatX, int sizeX, int sizeY);
  extern void cdecl L256_memset (void *dst, int value, int count);

  }

  #else

  void L256_movWin2_ (void *offset1, void *offset2, void *win,
                      int srcFormatX, int sizeX, int sizeY)
  {
    for ( ; sizeY > 0; sizeY--)
    {
      _fmemcpy (MK_FP (SVGA_linearSeg, offset1) , win, sizeX);
      _fmemcpy (MK_FP (SVGA_linearSeg, offset2) , win, sizeX);
      * (int *) &offset1  += SVGA_virtualX;
      * (int *) &offset2  += SVGA_virtualX;
      * (int *) &win += srcFormatX;
    }
  }


  void L256_sprcpy (void far *dst, void *spr, int count)
  {
    int i;

    for (i = 0; i < count; i++)
    {
      if (((char *)spr) [i])
      {
        ((char far *)dst) [i] = ((char *)spr) [i];
      }
    }
  }

  void L256_movSpr2_ (void *offset1, void *offset2, void *win,
                      int srcFormatX, int sizeX, int sizeY)
  {
    for ( ; sizeY > 0; sizeY--)
    {
      L256_sprcpy (MK_FP (SVGA_linearSeg, offset1), win, sizeX);
      L256_sprcpy (MK_FP (SVGA_linearSeg, offset2), win, sizeX);
      * (int *) &offset1  += SVGA_virtualX;
      * (int *) &offset2  += SVGA_virtualX;
      * (int *) &win += srcFormatX;
    }
  }

  void L256_memset (void *dst, int value, int count)
  {
    _fmemset (MK_FP (SVGA_linearSeg, dst), value, count);
  }

  #endif

#endif

#include "l256c_.cpp"

void L256_clear (long color)
{
  int es;
  long count;
  void *dst;

  count = (long) SVGA_virtualX * SVGA_scanLines;
  dst = SVGA_linearOff;
  if (SVGA_linearSeg)
  {
    es = DPMI_getES ();
    DPMI_setES (SVGA_linearSeg);
    L256_memset (dst, color, count);
    DPMI_setES (es);
  }
  else
  {
    memset (dst, color, count);
  }
}

void L256_putPixel (int X, int Y, long C)
{
  char far *dst;

  if ((X < SVGA_virtualX) && (Y < SVGA_virtualY))
  {
    dst = ((char far *)SVGA_linearBuf + X + Y*SVGA_virtualX);
    *dst = C;
    if (SVGA_doubleBuffer)
    {
      *(int *) &dst += SVGA_virtual;
      *dst = C;
    }
  }
}

long L256_getPixel (int X, int Y)
{
  if ((X < SVGA_virtualX) && (Y < SVGA_virtualY))
  {
    return *(char far *) SVGA_linearBuf+ X + Y*SVGA_virtualX;
  }
  return 0;
}

/*
void L256_putPixel (int X, int Y, long C)
{
  char c;
  c = C;
  SVGA_putWin_ (&c, X, Y, 1, 1, 1);
}
*/

void L256_getWin (void *win, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  int ds;
  int virtualX;
  void *linearOff;

  if (win != NULL)
  {
    X += SVGA_curX;
    Y += SVGA_curY;
    SVGA_virtual2Real (&X, &Y);

    if (SVGA_linearSeg)
    {
      virtualX = SVGA_virtualX;
      linearOff = SVGA_linearOff;

      ds = DPMI_getDS ();
      DPMI_setDS (SVGA_linearSeg);

      LIN_movWin_ (win, formatX, (char *)linearOff+X+Y*virtualX,
                   virtualX, SIZE_X, SIZE_Y);
      DPMI_setDS (ds);
    }
    else
    {
      LIN_movWin_ (win, formatX, (char *)SVGA_linearOff+X+Y*SVGA_virtualX,
                   SVGA_virtualX, SIZE_X, SIZE_Y);
    }
  }
}



#ifndef __TURBOC__

void L256_putWin2 (void *win, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  unsigned voff;
  int sizeX, sizeY;
  int restX, restY;;
  int virtualX;
  int es;
  char *spr = (char *) win;
  char *linearOff;
  char *linearOff2;
  char *loff;
  char *loff2;

  if (SVGA_linearSeg)
  {
    es = DPMI_getES ();
    DPMI_setES (SVGA_linearSeg);
  }

  X += SVGA_curX;
  Y += SVGA_curY;
  SVGA_virtual2Real (&X, &Y);

  virtualX = SVGA_virtualX;

  voff = X+Y*virtualX;
  loff = (char *) SVGA_linearOff;
  loff2 = (char *) SVGA_linearOff2;
  linearOff = loff + voff;
  linearOff2 = loff2 + voff;

  sizeY = (SVGA_virtual-voff)/virtualX;

  if (sizeY > SIZE_Y)
  {
    L256_movWin2_ (linearOff, linearOff2, spr, formatX, SIZE_X, SIZE_Y);
  }
  else
  {
    if (sizeY)
    {
      L256_movWin2_ (linearOff, linearOff2, spr, formatX, SIZE_X, sizeY);
    }

    restY = SIZE_Y-sizeY-1;
    if (restY >= 0)
    {
      spr+=sizeY*formatX;
      voff+=sizeY*virtualX;
      linearOff+=sizeY*virtualX;
      linearOff2+=sizeY*virtualX;

      restX = SVGA_virtual - voff;

      sizeX = (restX > SIZE_X) ? SIZE_X : restX;

      L256_movWin2_ (linearOff, linearOff2, spr, sizeX, sizeX, 1);
      L256_movWin2_ (loff, loff2, spr+sizeX, SIZE_X-sizeX, SIZE_X-sizeX, 1);

      if (restY)
      {
        spr += formatX;
        loff += virtualX-restX;
        loff2 += virtualX-restX;

        L256_movWin2_ (loff, loff2, spr, formatX, SIZE_X, restY);
      }
    }
  }

  if (SVGA_linearSeg)
  {
    DPMI_setES (es);
  }

}

void L256_putSpr2 (void *win, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  unsigned voff, soff;
  int sizeX, sizeY;
  int restX, restY;;
  int virtualX;
  int es;

  if (SVGA_linearSeg)
  {
    es = DPMI_getES ();
    DPMI_setES (SVGA_linearSeg);
  }

  X += SVGA_curX;
  Y += SVGA_curY;
  SVGA_virtual2Real (&X, &Y);

  virtualX = SVGA_virtualX;

  voff = X+Y*virtualX;

  sizeY = (SVGA_virtual-voff)/virtualX;

  if (sizeY > SIZE_Y)
  {
    L256_movSpr2_ ((char *)SVGA_linearOff+voff,
                   (char *)SVGA_linearOff2+voff,
                   (char *)win, formatX, SIZE_X, SIZE_Y);
  }
  else
  {
    if (sizeY)
    {
      L256_movSpr2_ ((char *)SVGA_linearOff+voff,
                     (char *)SVGA_linearOff2+voff,
                     (char *)win, formatX, SIZE_X, sizeY);
    }

    restY = SIZE_Y-sizeY-1;
    if (restY >= 0)
    {
      soff=sizeY*formatX;
      voff+=sizeY*virtualX;

      restX = SVGA_virtual - voff;

      sizeX = (restX > SIZE_X) ? SIZE_X : restX;

      L256_movSpr2_ ((char *)SVGA_linearOff+voff, (char *)SVGA_linearOff2+voff, (char *)win+soff, sizeX, sizeX, 1);
      L256_movSpr2_ ((char *)SVGA_linearOff, (char *) SVGA_linearOff2, (char *) win+soff+sizeX, SIZE_X-sizeX, SIZE_X-sizeX, 1);

      if (restY)
      {
        soff += formatX;
        voff = virtualX-restX;

        L256_movSpr2_ ((char *)SVGA_linearOff+voff,
                       (char *)SVGA_linearOff2+voff,
                       ((char *)win)+soff, formatX, SIZE_X, restY);
      }
    }
  }

  if (SVGA_linearSeg)
  {
    DPMI_setES (es);
  }

}

#endif



void L256_putWin (void *win, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  int es;

  X += SVGA_curX;
  Y += SVGA_curY;
  SVGA_virtual2Real (&X, &Y);

  if (SVGA_linearSeg)
  {
    es = DPMI_getES ();
    DPMI_setES (SVGA_linearSeg);

    LIN_movWin_ ((char *)SVGA_linearOff+X+Y*SVGA_virtualX, SVGA_virtualX,
                 win, formatX, SIZE_X, SIZE_Y);

    DPMI_setES (es);
  }
  else
  {
    LIN_movWin_ ((char *)SVGA_linearOff+X+Y*SVGA_virtualX, SVGA_virtualX,
                 win, formatX, SIZE_X, SIZE_Y);
  }
}

void L256_putSpr (void *win, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  int es;

  X += SVGA_curX;
  Y += SVGA_curY;
  SVGA_virtual2Real (&X, &Y);

  if (SVGA_linearSeg)
  {
    es = DPMI_getES ();
    DPMI_setES (SVGA_linearSeg);

    LIN_movSpr_ ((char *)SVGA_linearOff+X+Y*SVGA_virtualX, SVGA_virtualX,
                 win, formatX, SIZE_X, SIZE_Y);

    DPMI_setES (es);
  }
  else
  {
    LIN_movSpr_ ((char *)SVGA_linearOff+X+Y*SVGA_virtualX, SVGA_virtualX,
                 win, formatX, SIZE_X, SIZE_Y);

  }
}





























/*


#ifndef __TURBOC__


void L256_putWin2 (void *win, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  unsigned voff;
  int sizeX, sizeY;
  int restX, restY;;
  int virtualX;
  int es;
  char *spr = (char *) win;
  char *linearOff;
  char *linearOff2;
  char *loff;
  char *loff2;

  X += SVGA_curX;
  Y += SVGA_curY;
  SVGA_virtual2Real (&X, &Y);

  virtualX = SVGA_virtualX;

  voff = X+Y*virtualX;
  loff = (char *) SVGA_linearOff;
  loff2 = (char *) SVGA_linearOff2;
  linearOff = loff + voff;
  linearOff2 = loff2 + voff;

  sizeY = (SVGA_virtual-voff)/virtualX;

  if (sizeY > SIZE_Y)
  {
    L256C_putWin2 (linearOff, linearOff2, spr, formatX, SIZE_X, SIZE_Y);
  }
  else
  {
    if (sizeY)
    {
      L256C_putWin2 (linearOff, linearOff2, spr, formatX, SIZE_X, sizeY);
    }

    restY = SIZE_Y-sizeY-1;
    if (restY >= 0)
    {
      spr+=sizeY*formatX;
      voff+=sizeY*virtualX;
      linearOff+=sizeY*virtualX;
      linearOff2+=sizeY*virtualX;

      restX = SVGA_virtual - voff;

      sizeX = (restX > SIZE_X) ? SIZE_X : restX;

      L256C_putWin2 (linearOff, linearOff2, spr, sizeX, sizeX, 1);
      L256C_putWin2 (loff, loff2, spr+sizeX, SIZE_X-sizeX, SIZE_X-sizeX, 1);

      if (restY)
      {
        spr += formatX;
        loff += virtualX-restX;
        loff2 += virtualX-restX;

        L256C_putWin2 (loff, loff2, spr, formatX, SIZE_X, restY);
      }
    }
  }

  #endif
}

void L256_putSpr2 (void *win, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  unsigned voff, soff;
  int sizeX, sizeY;
  int restX, restY;;
  int virtualX;
  int es;

  X += SVGA_curX;
  Y += SVGA_curY;
  SVGA_virtual2Real (&X, &Y);

  virtualX = SVGA_virtualX;

  voff = X+Y*virtualX;

  sizeY = (SVGA_virtual-voff)/virtualX;

  if (sizeY > SIZE_Y)
  {
    L256C_putSpr2 ((char *)SVGA_linearOff+voff,
                   (char *)SVGA_linearOff2+voff,
                   (char *)win, formatX, SIZE_X, SIZE_Y);
  }
  else
  {
    if (sizeY)
    {
      L256C_putSpr2 ((char *)SVGA_linearOff+voff,
                     (char *)SVGA_linearOff2+voff,
                     (char *)win, formatX, SIZE_X, sizeY);
    }

    restY = SIZE_Y-sizeY-1;
    if (restY >= 0)
    {
      soff=sizeY*formatX;
      voff+=sizeY*virtualX;

      restX = SVGA_virtual - voff;

      sizeX = (restX > SIZE_X) ? SIZE_X : restX;

      L256C_putSpr2 ((char *)SVGA_linearOff+voff, (char *)SVGA_linearOff2+voff, (char *)win+soff, sizeX, sizeX, 1);
      L256C_putSpr2 ((char *)SVGA_linearOff, (char *) SVGA_linearOff2, (char *) win+soff+sizeX, SIZE_X-sizeX, SIZE_X-sizeX, 1);

      if (restY)
      {
        soff += formatX;
        voff = virtualX-restX;

        L256C_putSpr2 ((char *)SVGA_linearOff+voff,
                       (char *)SVGA_linearOff2+voff,
                       ((char *)win)+soff, formatX, SIZE_X, restY);
      }
    }
  }

  #endif
}

#endif


void L256_putWin (void *win, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  int es;

  X += SVGA_curX;
  Y += SVGA_curY;
  SVGA_virtual2Real (&X, &Y);

  L256C_putWin ((char *)SVGA_linearOff+X+Y*SVGA_virtualX, SVGA_virtualX,
                 win, formatX, SIZE_X, SIZE_Y);

}

void L256_putSpr (void *win, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  int es;

  X += SVGA_curX;
  Y += SVGA_curY;
  SVGA_virtual2Real (&X, &Y);

  L256C_putSpr ((char *)SVGA_linearOff+X+Y*SVGA_virtualX, SVGA_virtualX,
                 win, formatX, SIZE_X, SIZE_Y);
}

*/





/*
void L256_clear3 (long Color)
{
  int count;
  count = SVGA_virtualX * SVGA_scanLines;
  _fmemset (SVGA_linearBuf, Color, count);
}

void L256_clear2 (long color)
{
  int ds;
  int count;
  void *dst;

  count = SVGA_virtualX * SVGA_scanLines;
  dst = SVGA_linearOff;
  if (SVGA_linearSeg)
  {
    ds = DPMI_getDS ();
    DPMI_setDS (SVGA_linearSeg);
    memset (dst, color, count);
    DPMI_setDS (ds);
  }
  else
  {
    memset (dst, color, count);
  }
}
*/



/*
static void L256_copySpr2 (void *Win, int voffset, int limit, int sizeX, int sizeY, int formatX)
{
  int i;

  for ( ; sizeY > 0; sizeY--)
  {
    for (i = 0; i < sizeX; i++)
    {
      if (((char *)Win) [i])
      {
        ((char far *)SVGA_linearBuf) [voffset+i] =
        ((char far *)SVGA_linearBuf) [voffset+i+limit] = ((char *)Win) [i];
      }
    }
    voffset += SVGA_virtualX;
    *(int *)&Win += formatX;
  }
}
*/

/*
void L256C_putSpr2 (void *Win, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  int i,j;
  unsigned voffset;
  unsigned soffset;
  unsigned limit;
  int sizeY,restY;
  char c;

  if (SIZE_X)
  {
    X += SVGA_curX;
    Y += SVGA_curY;
    SVGA_virtual2Real (&X, &Y);

    voffset = X+Y*SVGA_virtualX;
    sizeY = (SVGA_virtual-voffset)/SVGA_virtualX;
    if (sizeY > SIZE_Y)
      sizeY = SIZE_Y;

    L256_copySpr2 ((char *)Win, voffset, SVGA_virtual, SIZE_X, sizeY, formatX);

    restY = SIZE_Y-sizeY-1;
    if (restY >= 0)
    {
      limit=SVGA_virtual;
      voffset+=sizeY*SVGA_virtualX;
      soffset=sizeY*formatX;

      for (i = 0; i < SIZE_X; i++)
      {
        if (voffset >= limit)
        {
          voffset-=limit;
        }
        c = ((char *)Win) [soffset];
        if (c)
        {
          ((char far *)SVGA_linearBuf) [voffset] =
          ((char far *)SVGA_linearBuf) [voffset+limit] = c;
        }
        voffset++;
        soffset++;
      }
      soffset += formatX-SIZE_X;
      voffset += SVGA_virtualX-SIZE_X;
      if (voffset >= limit)
      {
        voffset -= limit;
      }
      L256_copySpr2 (((char *)Win)+soffset, voffset, limit, SIZE_X, restY, formatX);
    }
  }
}
*/
/*
extern void L256_copySprLine2 (void);
#pragma aux L256_copySprLine2 = \
    "push  ebp"\
  "Main:"\
    "lodsd"\
  "C1:"\
    "or    eax,eax"\
    "je    LoopEnd"\
    "xor   edx,edx"\
    "or    al,al"\
    "jne   C2"\
    "dec   dl"\
  "C2:"\
    "or    ah,ah"\
    "jne   C3"\
    "dec   dh"\
  "C3:"\
    "ror   edx,16"\
  "Rot:"\
    "ror   eax,16"\
    "or    al,al"\
    "jne   C4"\
    "dec   dl"\
  "C4:"\
    "or    ah,ah"\
    "jne   DeRot"\
    "dec   dh"\
  "DeRot:"\
    "ror   eax,16"\
  "Save:"\
    "ror   edx,16"\
    "mov   ebp,eax"\
    "and   es:[edi],edx"\
    "or    es:[edi],eax"\
\
    "and   es:[ebx],edx"\
    "or    es:[edi],eax"\
  "LoopEnd:"\
    "add   edi,4"\
    "add   ebx,4"\
    "dec   ecx"\
    "jne   Main"\
    "pop   ebp"\
\
   modify [eax ebx ecx edx esi edi]


void L256_copySprLine2 (void)
{
  L256_copySprLine2 ();
}
#pragma aux L256_copySprLine2 modify [eax ebx ecx edx esi edi]

static void L256_copySpr2 (char *offset1, char *offset2, char *win, int sizeX, int sizeY, int formatX)
{
  int dstPlus;
  int sprPlus;
  int restX;

  void _LIN_putspr2cpy (void *offset1, void *offset2, void *win);
    #pragma aux _LIN_putspr2cpy =    \
                            \
  \
  "LoopY:"\
    "mov   ecx,sizeX"\
    "jecxz Rest"\
    "call  L256_copySprLine2"\
  "Rest:"\
    "mov   ecx,restX"\
    "jecxz EndRest"\
    "mov   eax,[esi]"\
  "C:"\
  \
    "or    al,al"\
    "je    Continue"\
    "mov   es:[edi],al"\
    "mov   es:[ebx],al"\
  "Continue:"\
    "shr   eax,8"\
    "inc   edi"\
    "inc   ebx"\
    "dec   ecx"\
    "jne   C"\
  "EndRest:"\
    "add   edi,dstPlus"\
    "add   ebx,dstPlus"\
    "add   esi,sprPlus"\
    "dec   sizeY"\
    "jne   LoopY"\
  \
    parm [edi] [ebx] [esi]\
    modify [eax ebx ecx edx edi esi]

  restX = sizeX & 3;
  sprPlus = formatX - sizeX + restX;
  dstPlus = SVGA_virtualX - sizeX;
  sizeX = sizeX >> 2;
  _LIN_putspr2cpy (offset1, offset2, win);
}

static void L256_copySpr2new (char *offset1, char *offset2, char *win, int sizeX, int sizeY, int formatX)
{
  LIN_movSpr_ (offset1, SVGA_virtualX, win, formatX, sizeX, sizeY);
  LIN_movSpr_ (offset2, SVGA_virtualX, win, formatX, sizeX, sizeY);
}

void L256_memset (void *buf, int value, int count);
  #pragma aux L256_memset = \
  "  mov    ah,al"\
  "  mov    edx,eax"\
  "  shr    edx,16"\
  "  or     eax,edx"\
  "  mov    edx,ecx"\
  "  shr    ecx,2"\
  "  repne  stosd"\
  "  mov    ecx,edx"\
  "  and    ecx,3"\
  "  repne  stosb"\
\
  parm [edi] [eax] [ecx]\
  modify [edi eax ecx edx]\
*/
/*
void L256_copyLine2 (char *offset1, char *offset2, char *win, int count);

  #pragma aux L256_copyLine2 = \
\
  "    mov     edx,ecx"\
  "    shr     ecx,2"\
  "    je      Rest"\
\
  "Loop1:"\
  "    lodsd"\
  "    mov     es:[ebx],eax"\
  "    add     ebx,4"\
  "    dec     ecx"\
  "    stosd"\
  "    jne     Loop1"\
\
  "Rest:"\
  "    and     edx,3"\
  "    je      End"\
\
  "Loop2:"\
  "    lodsb"\
  "    mov     es:[ebx],al"\
  "    inc     ebx"\
  "    dec     edx"\
  "    stosb"\
  "    jne     Loop2"\
  "End:"\
\
  parm [edi] [ebx] [esi] [ecx]\
  modify [eax ebx ecx edx esi edi]\

static void L256_copyWin2 (char *offset1, char *offset2, char *win, int sizeX, int sizeY, int formatX)
{
  int videoPlus;
  int winPlus;

  extern void L256_puteWin2 (char *offset1, char *offset2, char *win1, int sizeX, int sizeY);
  #pragma aux L256_puteWin2 = \
\
  "YLoop:"\
  "    push    ecx"\
  "    mov     ecx,edx"\
  "    shr     ecx,2"\
  "    je      XRest"\
\
  "XLoop:"\
  "    lodsd"\
  "    mov     es:[ebx],eax"\
  "    add     ebx,4"\
  "    dec     ecx"\
  "    stosd"\
  "    jne     XLoop"\
\
  "XRest:"\
  "    mov     ecx,edx"\
  "    and     ecx,3"\
  "    je      YNew"\
\
  "XLoop2:"\
  "    lodsb"\
  "    mov     es:[ebx],al"\
  "    inc     ebx"\
  "    dec     ecx"\
  "    stosb"\
  "    jne     XLoop2"\
\
  "YNew:"\
  "    add     edi,videoPlus"\
  "    add     ebx,videoPlus"\
  "    add     esi,winPlus"\
  "    pop     ecx"\
  "    dec     ecx"\
  "    jne     YLoop"\
\
  parm [edi] [ebx] [esi] [edx] [ecx]\
  modify [eax ebx ecx edx esi edi]\

  extern void L256_puteWin20 (char *offset1, char *offset2, char *win1, int sizeY);
  #pragma aux L256_puteWin20 = \
\
  "YLoop:"\
  "    mov     ecx,sizeX"\
\
  "XLoop:"\
  "    lodsd"\
  "    mov     es:[ebx],eax"\
  "    add     ebx,4"\
  "    dec     ecx"\
  "    stosd"\
  "    jne     XLoop"\
\
  "YNew:"\
  "    add     edi,videoPlus"\
  "    add     ebx,videoPlus"\
  "    add     esi,winPlus"\
  "    dec     edx"\
  "    jne     YLoop"\
  "End:"\
\
  parm [edi] [ebx] [esi] [edx]\
  modify [eax ebx ecx edx esi edi]\

  extern void L256_puteWin21 (char *offset1, char *offset2, char *win1, int sizeY);
  #pragma aux L256_puteWin21 = \
\
  "YLoop:"\
  "    mov     ecx,sizeX"\
\
  "XLoop:"\
  "    lodsd"\
  "    mov     es:[ebx],eax"\
  "    add     ebx,4"\
  "    dec     ecx"\
  "    stosd"\
  "    jne     XLoop"\
\
  "    lodsb"\
  "    mov     es:[ebx],al"\
  "    inc     ebx"\
  "    stosb"\
\
  "YNew:"\
  "    add     edi,videoPlus"\
  "    add     ebx,videoPlus"\
  "    add     esi,winPlus"\
  "    dec     edx"\
  "    jne     YLoop"\
\
  "End:"\
\
  parm [edi] [ebx] [esi] [edx]\
  modify [eax ebx ecx edx esi edi]\

  extern void L256_puteWin22 (char *offset1, char *offset2, char *win1, int sizeY);
  #pragma aux L256_puteWin22 = \
\
  "YLoop:"\
  "    mov     ecx,sizeX"\
\
  "XLoop:"\
  "    lodsd"\
  "    mov     es:[ebx],eax"\
  "    add     ebx,4"\
  "    dec     ecx"\
  "    stosd"\
  "    jne     XLoop"\
  "    lodsw"\
  "    mov     es:[ebx],ax"\
  "    inc     ebx"\
  "    inc     ebx"\
  "    stosw"\
\
  "YNew:"\
  "    add     edi,videoPlus"\
  "    add     ebx,videoPlus"\
  "    add     esi,winPlus"\
  "    dec     edx"\
  "    jne     YLoop"\
  "End:"\
\
  parm [edi] [ebx] [esi] [edx]\
  modify [eax ebx ecx edx esi edi]\

  extern void L256_puteWin23 (char *offset1, char *offset2, char *win1, int sizeY);
  #pragma aux L256_puteWin23 = \
\
  "YLoop:"\
  "    mov     ecx,sizeX"\
\
  "XLoop:"\
  "    lodsd"\
  "    mov     es:[ebx],eax"\
  "    add     ebx,4"\
  "    dec     ecx"\
  "    stosd"\
  "    jne     XLoop"\
  "    lodsw"\
  "    stosw"\
  "    mov     es:[ebx],ax"\
  "    inc     ebx"\
  "    inc     ebx"\
  "    lodsb"\
  "    mov     es:[ebx],al"\
  "    inc     ebx"\
  "    stosb"\
\
  "YNew:"\
  "    add     edi,videoPlus"\
  "    add     ebx,videoPlus"\
  "    add     esi,winPlus"\
  "    dec     edx"\
  "    jne     YLoop"\
  "End:"\
\
  parm [edi] [ebx] [esi] [edx]\
  modify [eax ebx ecx edx esi edi]\


  videoPlus = SVGA_virtualX - sizeX;
  winPlus = formatX - sizeX;

  switch (sizeX & 3)
  {
    case 0:
      sizeX = sizeX >> 2;
      if (sizeX)
      {
        L256_puteWin20 (offset1, offset2, win, sizeY);
      }
      break;
    case 1:
      sizeX = sizeX >> 2;
      if (sizeX)
      {
        L256_puteWin21 (offset1, offset2, win, sizeY);
      }
      break;
    case 2:
      sizeX = sizeX >> 2;
      if (sizeX)
      {
        L256_puteWin22 (offset1, offset2, win, sizeY);
      }
      break;

    case 3:
      sizeX = sizeX >> 2;
      if (sizeX)
      {
        L256_puteWin23 (offset1, offset2, win, sizeY);
      }
      break;

  }
}

static void L256_copyWin2 (char *offset1, char *offset2, char *win, int sizeX, int sizeY, int formatX)
{
  extern void L256_puteWin_ (char *offset, int dstFormatX, char *win, int sizeX, int sizeY, int formatX);
  #pragma aux L256_puteWin_ = \
  "    push    ebp"\
  "    sub     eax,ecx"\
  "    mov     ebp,eax"\
  "    mov     eax,ecx"\
  "    sub     edx,eax"\
  "YLoop:"\
  "    shr     ecx,2"\
  "    repne   movsd"\
  "    mov     ecx,eax"\
  "    and     ecx,3"\
  "    repne   movsb"\
  "    mov     ecx,eax"\
  "    add     edi,ebp"\
  "    add     esi,edx"\
  "    dec     ebx"\
  "    jne     YLoop"\
  "    pop     ebp"\
  \
  parm [edi] [eax] [esi] [ecx] [ebx] [edx]\
  modify [eax ebx ecx edx esi edi]\

  L256_puteWin_ (offset1, SVGA_virtualX, win, sizeX, sizeY, formatX);
  L256_puteWin_ (offset2, SVGA_virtualX, win, sizeX, sizeY, formatX);
}

#else

static void L256_copyWin2 (char *offset1, char *offset2, char *win, int sizeX, int sizeY, int formatX)
{

}
#endif
*/



/*
static void L256_copySpr2 (char *offset1, char *offset2, char *win, int sizeX, int sizeY, int formatX)
{
  int i;

  for ( ; sizeY > 0; sizeY--)
  {
    for (i = 0; i < sizeX; i++)
    {
      if (win [i])
      {
        offset1 [i] = offset2 [i] = win [i];
      }
    }
    *(int *) &offset1 += SVGA_virtualX;
    *(int *) &offset2 += SVGA_virtualX;
    *(int *) &win += formatX;
  }
}
*/

/*
void L256_putSpr2 (void *win, int X, int Y, int SIZE_X, int SIZE_Y, int formatX)
{
  int i,j;
  unsigned voff;
  unsigned soff;
  int sizeY,restY;
  char c;
  int es;

  if ((SIZE_X) && (SIZE_Y))
  {
    if (SVGA_linearSeg)
    {
      es = DPMI_getES ();
      DPMI_setES (SVGA_linearSeg);
    }

    X += SVGA_curX;
    Y += SVGA_curY;
    SVGA_virtual2Real (&X, &Y);

    voff = X+Y*SVGA_virtualX;
    sizeY = (SVGA_virtual-voff)/SVGA_virtualX;
    if (sizeY > SIZE_Y)
      sizeY = SIZE_Y;

//    L256_copySpr2 ((char *)SVGA_linearOff+voff, (char *)SVGA_linearOff+voff+SVGA_virtual, (char *) win, SIZE_X, sizeY, formatX);

    restY = SIZE_Y-sizeY-1;
    if (restY >= 0)
    {
      voff+=sizeY*SVGA_virtualX;
      soff=sizeY*formatX;

      for (i = 0; i < SIZE_X; i++)
      {
        if (voff >= SVGA_virtual)
        {
          voff-=SVGA_virtual;
        }
        c = ((char *)win) [soff];
        if (c)
        {
          ((char far *)SVGA_linearBuf) [voff] =
          ((char far *)SVGA_linearBuf) [voff+SVGA_virtual] = c;
        }
        voff++;
        soff++;
      }

      soff += formatX-SIZE_X;
      voff += SVGA_virtualX-SIZE_X;
      if (voff >= SVGA_virtual)
      {
        voff -= SVGA_virtual;
      }
//      L256_copySpr2 ((char *)SVGA_linearOff+voff, (char *)SVGA_linearOff+voff+SVGA_virtual, ((char *)win)+soff, SIZE_X, restY, formatX);
    }

    if (SVGA_linearSeg)
    {
      DPMI_setES (es);
    }
  }
}
  */

