
#include <stdio.h>
#include <dos.h>
#include <string.h>

#include "lzw.h"
#include "cgdk.h"
#include "timer.h"

#define LZWF_ERR_NONE 0
#define LZWF_ERR_HELP 3

static void mainHelp (void)
{
  printf ("Syntax is LZWF <compressed_file> <uncompressed_file> [-d]\n");
}

static int mainCompress (TLZW *lzw, const char *input, const char *output)
{
  int FInit;

  TIMER_start ();
  LZW_compressInit (lzw);
  FInit = LZWF_init (lzw, input, output);
  if (FInit == 0)
  {
    LZW_run (lzw);
    LZWF_done (lzw);
    return LZWF_ERR_NONE;
  }
  else
  {
    return FInit;
  }
}

static int mainUncompress (TLZW *lzw, const char *input, const char *output)
{
  int FInit;

  TIMER_start ();
  LZW_uncompressInit (lzw);
  FInit = LZWF_init (lzw, input, output);
  if (FInit == 0)
  {
    LZW_run (lzw);
    return LZWF_ERR_NONE;
  }
  else
  {
    return FInit;
  }
}

static void mainInfo (TLZW *lzw)
{
  float ratio;

  ratio = (float) (100*lzw->bytesWritten)/lzw->bytesRead;
  printf ("\n"
          "Bytes read    : %li\n"
          "Bytes written : %li\n"
          "Ratio         : %.2f%\n"
          "\n",
          lzw->bytesRead, lzw->bytesWritten, ratio);
  printf ("Time          : %li ms\n", (long) ((1000*TIMER_time) / TIMER_FREQ));
}

int main (void)
{
  int exitCode;
  TLZW *lzw;

  printf ("\n"
          "LZWF - example lzw file compressor\n"
          "\n%s\n", CGDK_copyright);

//  CGDK_init ();
  TIMER_init (TIMER_OLDFREQ_DEF);

  LZW_init ();

  lzw = LZW_new ();
  if (lzw)
  {
    switch (_argc)
    {
      case 3:
      {
        exitCode = mainCompress (lzw, _argv [2], _argv [1]);
        break;
      }
      case 4:
      {
        if ( (strcmpi (_argv [3], "-d") == 0) || (strcmp (_argv [3], "/d") == 0))
        {
          exitCode = mainUncompress (lzw, _argv [1], _argv [2]);
        }
        else
        {
          exitCode = LZWF_ERR_HELP;
          break;
        }
        break;
      }
      default:
      {
        exitCode = LZWF_ERR_HELP;
        break;
      }
    }

    switch (exitCode)
    {
      case LZWF_ERR_NONE:
      {
        mainInfo (lzw);
        break;
      }
      case LZWF_ERR_HELP:
      {
        mainHelp ();
        break;
      }
      case LZWF_ERR_INPUT:
      {
        CGDK_exit ("Error reading input file.\n", EXIT_FAILURE);
        break;
      }
      case LZWF_ERR_OUTPUT:
      {
        CGDK_exit ("Error writing output file.\n", EXIT_FAILURE);
        break;
      }
    }

    LZW_free (lzw);

    LZW_done ();
  }
  else
  {
    CGDK_exit ("Error allocating LZW handler.\n", EXIT_FAILURE);
  }

  TIMER_done ();

  return exitCode;
}
