
#include <stdlib.h>

#include "array.h"
#include "lin.h"
#include "svga.h"
#include "mem.h"
#include "vsb.h"
#include "cgdk.h"
#include "svga.h"

#include "lin.h"
#include "mir.h"

int MIRL_max;

typedef TMIR *TMIRL [MIRL_MAX];
//static char *vbuf;

static TMIRL *MIRL;

static void MIRL_done (void)
{
//  if (vbuf != NULL)
//  {
//    _free (vbuf);
//    vbuf = NULL;
//  }

  if (MIRL != NULL)
  {
    _free (MIRL);
    MIRL = NULL;
  }
}



static void MIRL_init (int maxLayer)
{
  MIRL_done ();

  MIRL = (TMIRL *) _calloc (maxLayer, sizeof (TMIRL), NULL);
  if (MIRL == NULL)
  {
    CGDK_exit ("Not enough memory to allocate mirror layer buffer.\n", EXIT_FAILURE);
  }
  MIRL_max = maxLayer;
//  vbuf = (char *) _calloc (sizeX,sizeY);
}


static void MIRL_insert (TMIR *mir, int layer)
{
  TMIR lmir;

  if ((layer < MIRL_max) && (mir != NULL))
  {
    if ((layer > -1) && (mir->layer <= -1))
    {
      AL_insert ((long *) MIRL [layer], *(long *) &mir, MIRL_MAX);
      mir->layer = layer;
    }
  }
}

static void MIRL_delete (TMIR *mir)
{
  if (mir)
  {
    if (mir->layer > -1)
    {
      if ((mir->layer < MIRL_max))
      {
        AL_delete ((long *) MIRL [mir->layer], *(long *) &mir, MIRL_MAX);
        mir->layer = -1;
      }
    }
  }
}

static void MIRL_engine (char *buf, int x, int y,
                         int sizeX, int sizeY, TMIR *mir)
{
  int msizeX, msizeY;
  int sstartX, sstartY;
  int dstartX, dstartY;
  int mirsizeX;
  int mirsizeY;
  int mirx;
  int miry;

  mirx = mir->x;
  mirsizeX = mir->sizeX;
  if (mirx < x)
  {
    dstartX = 0;
    sstartX = (x - mirx);
    if ((mirsizeX - sstartX) < sizeX)
      msizeX = mirsizeX-sstartX;
    else
      msizeX = sizeX;
  }
  else
  {
    dstartX = mirx - x;
    sstartX = 0;
    if ((sizeX - dstartX) < mirsizeX)
      msizeX = sizeX-dstartX;
    else
      msizeX = mirsizeX;
  }
  if (msizeX > 0)
  {
    miry = mir->y;
    mirsizeY = mir->sizeY;
    if (miry < y)
    {
      dstartY = 0;
      sstartY = (y - miry);
      if ((mirsizeY - sstartY) < sizeY)
        msizeY = mirsizeY-sstartY;
      else
        msizeY = sizeY;
    }
    else
    {
      dstartY = mir->y - y;
      sstartY = 0;
      if ((sizeY - dstartY) < mirsizeY)
        msizeY = sizeY-dstartY;
      else
        msizeY = mirsizeY;
    }

    if (msizeY > 0)
    {
      LIN_movSpr_ (buf+dstartX+sizeX*(dstartY), sizeX,
                   mir->raw+sstartX+mirsizeX*(sstartY), mirsizeX,
                   msizeX, msizeY);
    }
  }
}

/*
  if (put)
  {
    MIRL_put_ (buf+dindex, mirspr+sindex, msizeX, msizeY, mirsizeX, sizeX);
  }
  else
  {
    MIRL_get_ (buf+dindex, mirspr+sindex, msizeX, msizeY, mirsizeX, sizeX);
  }*/

static void MIRL_get (TMIR *mir, void *Sprite,int X,int Y,
                           int Size_X, int Size_Y)
{
  int pos;
  int i;
  TMIR *nmir;
  TMIR **mirl;

  SVGA_getWin ((char *) Sprite, X, Y, Size_X, Size_Y, Size_X);

  if (mir->layer != -1)
  {
    for (i = 0; i < mir->layer; i++)
    {
      mirl = (TMIR * *) MIRL [i];
      for (pos = 0; (nmir = mirl [pos]) != NULL; pos++)
      {
        if ((nmir->status & MIR_STATUS_RESTORE) == 0)
        {
          MIRL_engine ((char *) Sprite, X, Y, Size_X, Size_Y, nmir);
        }
      }
    }
  }
}


static void MIRL_putWin (TMIR *mir, void *Sprite,int X,int Y,
                           int Size_X, int Size_Y)
{
  int pos;
  int i;
  TMIR *nmir;
  TMIR **mirl;

  if (mir->layer != -1)
  {
    for (i = mir->layer+1; i < MIRL_max; i++)
    {
      mirl = MIRL [i];
      for (pos = 0; (nmir = mirl [pos]) != NULL; pos++)
      {
        if ((nmir->status & MIR_STATUS_RESTORE) == 0)
        {
          MIRL_engine ((char *)Sprite, X, Y, Size_X, Size_Y, nmir); ///vbuf!
        }
      }
    }
  }
  SVGAC_putWinNoVSB (Sprite,  X, Y, Size_X, Size_Y, Size_X);
}

static void MIRL_putSpr (TMIR *mir, void *Sprite,int X,int Y,
                           int Size_X, int Size_Y)
{
  int pos;
  int i;
  TMIR *nmir;
  TMIR **mirl;

  if (mir->layer != -1)
  {
    for (i = mir->layer+1; i < MIRL_max; i++)
    {
      mirl = MIRL [i];
      for (pos = 0; (nmir = mirl [pos]) != NULL; pos++)
      {
        if ((nmir->status & MIR_STATUS_RESTORE) == 0)
        {
          MIRL_engine ((char *)Sprite, X, Y, Size_X, Size_Y, nmir); ///vbuf!
        }
      }
    }
  }
  SVGAC_putSprNoVSB (Sprite,  X, Y, Size_X, Size_Y, Size_X);
}

void MIRL_updateCurPosAll (void)
{
  int pos;
  int i;
  TMIR **mirl;
  TMIR *nmir;

  for (i = 0; i < MIRL_max; i++)
  {
    mirl = MIRL [i];
    for (pos = 0; (nmir = mirl [pos]) != NULL; pos++)
    {
      MIR_updateCurPos (nmir);
    }
  }
}

void MIRL_restoreAll (void)
{
  int pos;
  int i;
  TMIR *nmir;
  TMIR **mirl;

  SVGAC_flush ();
  for (i = 0; i < MIRL_max; i++)
  {
    mirl = MIRL [i];
    for (pos = 0; (nmir = mirl [pos]) != NULL; pos++)
    {
      if ( (nmir->status & MIR_STATUS_RESTORE) == 0 )
      {
        MIR_shade (nmir);
        SVGA_putSpr (nmir->raw, nmir->x, nmir->y, nmir->sizeX, nmir->sizeY, nmir->sizeX);
        nmir->status = MIR_STATUS_RESTORE;
      }
    }
  }
}

/*
static void MIRL_put_ (char *buf, char *mirspr,
                       int msizeX, int msizeY,
                       int mirsizeX,int sizeX)
{
  int i;

  for ( ; msizeY > 0; msizeY--)
  {
    for (i = 0; i < msizeX; i++)
    {
      if (mirspr [i] != 0)
      {
        buf [i] = mirspr [i]; ///
      }
    }
    *(int *)&mirspr+=mirsizeX;
    *(int *)&buf+=sizeX;
  }
}

static void MIRL_get_ (char *buf, char *mirspr,
                       int msizeX, int msizeY,
                       int mirsizeX,int sizeX)
{
  int i;

  for ( ; msizeY > 0; msizeY--)
  {
    for (i = 0; i < msizeX; i++)
    {
      if (mirspr [i] != 0)
      {
        buf [i] = mirspr [i];
      }
    }
    *(int *)&mirspr+=mirsizeX;
    *(int *)&buf+=sizeX;
  }
}
*/
