
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "wait.h"
#include "cgdk.h"
#include "dpmi.h"
#include "plasma.h"


/********** you are recommended to modify only this code **********/
/******************************************************************/

#include "plg000.h"

#define P_LABEL "\n"\
                "This is the default plasma image for 320 x 200, 256 color resolution\n"\
                "Copyright (C) 1994-1997 Cybertek Corp.\n"

#define P_BYTESPERPIXEL 1
#define P_MOVE_STEPSMAX 10000
#define P_COL_STEPSMAX 10000
#define P_VIDEO_MAXX 320
#define P_VIDEO_MAXY 200
#define P_POOL_MAXX 1.6 * P_VIDEO_MAXX
#define P_POOL_MAXY 1.6 * P_VIDEO_MAXY
#define P_POOL_DIFX (P_POOL_MAXX - P_VIDEO_MAXX)/2
#define P_POOL_DIFY (P_POOL_MAXY - P_VIDEO_MAXY)/2
#define P_COL_MAX 255

/******************************************************************/
/******************************************************************/



static void PLASMA_genValues (TPLASMA_image *pi, int basecolor,
                              void (*waitFunc)(long,long))
{
  float x,y,step;
  long  i,j;

  /* First generate the plasma map.  This is just an
     function of x and y which gives a smooth surface */

  for (i=0,y=0;y<P_POOL_MAXY;y++)
  {
    for (x=0;x<P_POOL_MAXX;x++,i++)
    {

      /********** you are recommended to modify only this code **********/
      /******************************************************************/
      ((char *) pi->pool) [i] =

      basecolor-1 + ((P_COL_MAX/4)+(P_COL_MAX/25)*
      (sin(x/30) + cos(y/46) +
      cos(x/37) + sin(y/70) +
      sin((x+y)/43) +
      cos(hypot(256-x,150-y)/20)));
      /******************************************************************/
      /******************************************************************/
    }
    WAIT_func (waitFunc, y, P_POOL_MAXY, 0, 20, 50);
  }

  /* Then moves for two pointers */

  for (i=0,step=0;step<P_MOVE_STEPSMAX;step++,i++)
  {
    /********** you are recommended to modify only this code **********/
    /******************************************************************/

    pi->move [2*i] =

    P_POOL_DIFX+(P_POOL_DIFX-1)*cos(step/32)+P_POOL_MAXX*
    (long)(P_POOL_DIFY+(P_POOL_DIFY-1)*sin(step/16));


    pi->move [2*i+1] =

    P_POOL_DIFX+(P_POOL_DIFX-1)*sin(step/21)+P_POOL_MAXX
    *(long)(P_POOL_DIFY+(P_POOL_DIFY-1)*cos(step/24));
    /******************************************************************/
    /******************************************************************/

    if (i<P_COL_MAX)
    {
      ((long *) pi->col) [i] = 0;
    }
    else
    {
      pi->col [i].r = sin(step/20)*sin(step/15)*124+124;
      pi->col [i].g = sin(step/35)*sin(step/22)*124+124;
      pi->col [i].b = sin(step/13)*sin(step/30)*124+124;
      pi->col [i].a = 0;
    /******************************************************************/
    /******************************************************************/
    }
    WAIT_func (waitFunc, i, P_MOVE_STEPSMAX, 20, 30, 50);
  }
}




extern "C" TPLASMA_image *PLASMA_CALL (int basecolor,void (*waitFunc)(long,long))
{
  TPLASMA_image *pi;

  if ((pi = (TPLASMA_image *) _calloc (sizeof (TPLASMA_image), 1, NULL)) == NULL)
  {
    return NULL;
  }

  strcpy (pi->desc, PLASMA_DESC);
  strcpy (pi->label, P_LABEL);
  pi->id = PLASMA_ID;
  pi->version = PLASMA_VERSION;
  pi->bytesPerPixel = P_BYTESPERPIXEL;
  pi->videoSizeX = P_VIDEO_MAXX;
  pi->videoSizeY = P_VIDEO_MAXY;

  pi->pool = (long *) _alloc (P_POOL_MAXX*P_POOL_MAXY*P_BYTESPERPIXEL);
  if (pi->pool == NULL)
  {
    _free (pi);
    return NULL;
  }
  pi->poolSizeX = P_POOL_MAXX;
  pi->poolSizeY = P_POOL_MAXY;

  pi->move = (long *) _alloc (P_MOVE_STEPSMAX*2*sizeof (long));
  if (pi->move == NULL)
  {
    _free (pi->pool);
    _free (pi);
    return NULL;
  }

  pi->moveStepsMax = P_MOVE_STEPSMAX;
  pi->moveStep = 0;

  pi->col = (TRGBColor *) _alloc (P_COL_STEPSMAX*sizeof (TRGBColor));
  if (pi->col == NULL)
  {
    _free (pi->col);
    _free (pi->pool);
    _free (pi);
    return NULL;
  }

  pi->colBase = basecolor;
  pi->colMax = P_COL_MAX;
  pi->colStepsMax = P_COL_STEPSMAX;
  pi->colStep = 0;

  PLASMA_genValues (pi, basecolor, waitFunc);

  return pi;
}
